@@ -3,7 +3,7 @@ use cargo::util::{CliResult, Config};
 use cargo::util::important_paths::{find_root_manifest_for_wd};
 
 #[derive(RustcDecodable)]
-struct Options {
+pub struct Options {
     arg_opts: Vec<String>,
     flag_target: Option<String>,
     flag_features: Vec<String>,
@@ -11,9 +11,9 @@ struct Options {
     flag_manifest_path: Option<String>,
     flag_no_default_features: bool,
     flag_open: bool,
-    flag_verbose: bool,
+    flag_verbose: Option<bool>,
     flag_release: bool,
-    flag_quiet: bool,
+    flag_quiet: Option<bool>,
     flag_color: Option<String>,
     flag_package: Option<String>,
     flag_lib: bool,
@@ -62,8 +62,9 @@ the `cargo help pkgid` command.
 ";
 
 pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
-    try!(config.shell().set_verbosity(options.flag_verbose, options.flag_quiet));
-    try!(config.shell().set_color_config(options.flag_color.as_ref().map(|s| &s[..])));
+    try!(config.configure_shell(options.flag_verbose,
+                                options.flag_quiet,
+                                &options.flag_color));
 
     let root = try!(find_root_manifest_for_wd(options.flag_manifest_path,
                                               config.cwd()));
