@@ -79,7 +79,7 @@ impl<'cfg> PackageRegistry<'cfg> {
         PackageRegistry {
             sources: SourceMap::new(),
             source_ids: HashMap::new(),
-            overrides: vec![],
+            overrides: Vec::new(),
             config: config,
             locked: HashMap::new(),
         }
@@ -143,11 +143,9 @@ impl<'cfg> PackageRegistry<'cfg> {
         self.source_ids.insert(id.clone(), (id.clone(), kind));
     }
 
-    pub fn add_overrides(&mut self, ids: Vec<SourceId>) -> CargoResult<()> {
-        for id in ids.iter() {
-            try!(self.load(id, Kind::Override));
-        }
-        Ok(())
+    pub fn add_override(&mut self, id: &SourceId, source: Box<Source + 'cfg>) {
+        self.add_source(id, source, Kind::Override);
+        self.overrides.push(id.clone());
     }
 
     pub fn register_lock(&mut self, id: PackageId, deps: Vec<PackageId>) {
@@ -160,21 +158,16 @@ impl<'cfg> PackageRegistry<'cfg> {
 
     fn load(&mut self, source_id: &SourceId, kind: Kind) -> CargoResult<()> {
         (|| {
-            let mut source = source_id.load(self.config);
-
-            // Ensure the source has fetched all necessary remote data.
-            let p = profile::start(format!("updating: {}", source_id));
-            try!(source.update());
-            drop(p);
-
+            // Save off the source
+            let source = source_id.load(self.config);
             if kind == Kind::Override {
                 self.overrides.push(source_id.clone());
             }
-
-            // Save off the source
             self.add_source(source_id, source, kind);
 
-            Ok(())
+            // Ensure the source has fetched all necessary remote data.
+            let _p = profile::start(format!("updating: {}", source_id));
+            self.sources.get_mut(source_id).unwrap().update()
         }).chain_error(|| human(format!("Unable to update {}", source_id)))
     }
 
@@ -272,18 +265,16 @@ impl<'cfg> PackageRegistry<'cfg> {
 
 impl<'cfg> Registry for PackageRegistry<'cfg> {
     fn query(&mut self, dep: &Dependency) -> CargoResult<Vec<Summary>> {
-        let overrides = try!(self.query_overrides(dep));
+        let overrides = try!(self.query_overrides(&dep));
 
         let ret = if overrides.is_empty() {
             // Ensure the requested source_id is loaded
             try!(self.ensure_loaded(dep.source_id(), Kind::Normal));
-            let mut ret = Vec::new();
-            for (id, src) in self.sources.sources_mut() {
-                if id == dep.source_id() {
-                    ret.extend(try!(src.query(dep)).into_iter());
-                }
+
+            match self.sources.get_mut(dep.source_id()) {
+                Some(src) => try!(src.query(&dep)),
+                None => Vec::new(),
             }
-            ret
         } else {
             overrides
         };
