@@ -50,10 +50,24 @@ pub fn doc(manifest_path: &Path,
             }
         };
 
+        // Don't bother locking here as if this is getting deleted there's
+        // nothing we can do about it and otherwise if it's getting overwritten
+        // then that's also ok!
         let target_dir = options.compile_opts.config.target_dir(&package);
         let path = target_dir.join("doc").join(&name).join("index.html");
+        let path = path.into_path_unlocked();
         if fs::metadata(&path).is_ok() {
-            open_docs(&path);
+            let mut shell = options.compile_opts.config.shell();
+            match open_docs(&path) {
+                Ok(m) => try!(shell.status("Launching", m)),
+                Err(e) => {
+                    try!(shell.warn(
+                            "warning: could not determine a browser to open docs with, tried:"));
+                    for method in e {
+                        try!(shell.warn(format!("\t{}", method)));
+                    }
+                }
+            }
         }
     }
 
@@ -61,38 +75,40 @@ pub fn doc(manifest_path: &Path,
 }
 
 #[cfg(not(any(target_os = "windows", target_os = "macos")))]
-fn open_docs(path: &Path) {
-    // trying xdg-open
-    match Command::new("xdg-open").arg(path).status() {
-        Ok(_) => return,
-        Err(_) => ()
-    };
+fn open_docs(path: &Path) -> Result<&'static str, Vec<&'static str>> {
+    use std::env;
+    let mut methods = Vec::new();
+    // trying $BROWSER
+    match env::var("BROWSER"){
+        Ok(name) => match Command::new(name).arg(path).status() {
+            Ok(_) => return Ok("$BROWSER"),
+            Err(_) => methods.push("$BROWSER")
+        },
+        Err(_) => () // Do nothing here if $BROWSER is not found
+    }
 
-    // trying gnome-open
-    match Command::new("gnome-open").arg(path).status() {
-        Ok(_) => return,
-        Err(_) => ()
-    };
+    for m in ["xdg-open", "gnome-open", "kde-open"].iter() {
+        match Command::new(m).arg(path).status() {
+            Ok(_) => return Ok(m),
+            Err(_) => methods.push(m)
+        }
+    }
 
-    // trying kde-open
-    match Command::new("kde-open").arg(path).status() {
-        Ok(_) => return,
-        Err(_) => ()
-    };
+    Err(methods)
 }
 
 #[cfg(target_os = "windows")]
-fn open_docs(path: &Path) {
+fn open_docs(path: &Path) -> Result<&'static str, Vec<&'static str>> {
     match Command::new("cmd").arg("/C").arg("start").arg("").arg(path).status() {
-        Ok(_) => return,
-        Err(_) => ()
+        Ok(_) => return Ok("cmd /C start"),
+        Err(_) => return Err(vec!["cmd /C start"])
     };
 }
 
 #[cfg(target_os = "macos")]
-fn open_docs(path: &Path) {
+fn open_docs(path: &Path) -> Result<&'static str, Vec<&'static str>> {
     match Command::new("open").arg(path).status() {
-        Ok(_) => return,
-        Err(_) => ()
+        Ok(_) => return Ok("open"),
+        Err(_) => return Err(vec!["open"])
     };
 }
