@@ -1,6 +1,7 @@
-use std::io::prelude::*;
 use std::fs::{self, File};
-use std::path::{self, Path, PathBuf};
+use std::io::SeekFrom;
+use std::io::prelude::*;
+use std::path::{self, Path};
 
 use tar::{Archive, Builder, Header};
 use flate2::{GzBuilder, Compression};
@@ -8,16 +9,17 @@ use flate2::read::GzDecoder;
 
 use core::{SourceId, Package, PackageId};
 use sources::PathSource;
-use util::{self, CargoResult, human, internal, ChainError, Config};
+use util::{self, CargoResult, human, internal, ChainError, Config, FileLock};
 use ops;
 
 pub fn package(manifest_path: &Path,
                config: &Config,
                verify: bool,
                list: bool,
-               metadata: bool) -> CargoResult<Option<PathBuf>> {
-    let mut src = try!(PathSource::for_path(manifest_path.parent().unwrap(),
-                                            config));
+               metadata: bool) -> CargoResult<Option<FileLock>> {
+    let path = manifest_path.parent().unwrap();
+    let id = try!(SourceId::for_path(path));
+    let mut src = PathSource::new(path, &id, config);
     let pkg = try!(src.root_package());
 
     if metadata {
@@ -38,29 +40,37 @@ pub fn package(manifest_path: &Path,
 
     let filename = format!("{}-{}.crate", pkg.name(), pkg.version());
     let dir = config.target_dir(&pkg).join("package");
-    let dst = dir.join(&filename);
-    if fs::metadata(&dst).is_ok() {
-        return Ok(Some(dst))
-    }
+    let mut dst = match dir.open_ro(&filename, config, "packaged crate") {
+        Ok(f) => return Ok(Some(f)),
+        Err(..) => {
+            let tmp = format!(".{}", filename);
+            try!(dir.open_rw(&tmp, config, "package scratch space"))
+        }
+    };
 
     // Package up and test a temporary tarball and only move it to the final
     // location if it actually passes all our tests. Any previously existing
     // tarball can be assumed as corrupt or invalid, so we just blow it away if
     // it exists.
     try!(config.shell().status("Packaging", pkg.package_id().to_string()));
-    let tmp_dst = dir.join(format!(".{}", filename));
-    let _ = fs::remove_file(&tmp_dst);
-    try!(tar(&pkg, &src, config, &tmp_dst, &filename).chain_error(|| {
+    try!(dst.file().set_len(0));
+    try!(tar(&pkg, &src, config, dst.file(), &filename).chain_error(|| {
         human("failed to prepare local package for uploading")
     }));
     if verify {
-        try!(run_verify(config, &pkg, &tmp_dst).chain_error(|| {
+        try!(dst.seek(SeekFrom::Start(0)));
+        try!(run_verify(config, &pkg, dst.file()).chain_error(|| {
             human("failed to verify package tarball")
         }))
     }
-    try!(fs::rename(&tmp_dst, &dst).chain_error(|| {
-        human("failed to move temporary tarball into final location")
-    }));
+    try!(dst.seek(SeekFrom::Start(0)));
+    {
+        let src_path = dst.path();
+        let dst_path = dst.parent().join(&filename);
+        try!(fs::rename(&src_path, &dst_path).chain_error(|| {
+            human("failed to move temporary tarball into final location")
+        }));
+    }
     Ok(Some(dst))
 }
 
@@ -92,7 +102,7 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
         things.push_str(&missing.last().unwrap());
 
         try!(config.shell().warn(
-            &format!("warning: manifest has no {things}. \
+            &format!("manifest has no {things}. \
                     See http://doc.crates.io/manifest.html#package-metadata for more info.",
                     things = things)))
     }
@@ -102,26 +112,17 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
 fn tar(pkg: &Package,
        src: &PathSource,
        config: &Config,
-       dst: &Path,
+       dst: &File,
        filename: &str) -> CargoResult<()> {
-    if fs::metadata(&dst).is_ok() {
-        bail!("destination already exists: {}", dst.display())
-    }
-
-    try!(fs::create_dir_all(dst.parent().unwrap()));
-
-    let tmpfile = try!(File::create(dst));
-
     // Prepare the encoder and its header
     let filename = Path::new(filename);
     let encoder = GzBuilder::new().filename(try!(util::path2bytes(filename)))
-                                  .write(tmpfile, Compression::Best);
+                                  .write(dst, Compression::Best);
 
     // Put all package files into a compressed archive
     let mut ar = Builder::new(encoder);
     let root = pkg.root();
     for file in try!(src.list_files(pkg)).iter() {
-        if &**file == dst { continue }
         let relative = util::without_prefix(&file, &root).unwrap();
         try!(check_filename(relative));
         let relative = try!(relative.to_str().chain_error(|| {
@@ -172,11 +173,13 @@ fn tar(pkg: &Package,
     Ok(())
 }
 
-fn run_verify(config: &Config, pkg: &Package, tar: &Path)
+fn run_verify(config: &Config,
+              pkg: &Package,
+              tar: &File)
               -> CargoResult<()> {
     try!(config.shell().status("Verifying", pkg));
 
-    let f = try!(GzDecoder::new(try!(File::open(tar))));
+    let f = try!(GzDecoder::new(tar));
     let dst = pkg.root().join(&format!("target/package/{}-{}",
                                        pkg.name(), pkg.version()));
     if fs::metadata(&dst).is_ok() {
