@@ -1,14 +1,12 @@
 use std::fmt::{self, Debug, Formatter};
 use std::hash::{Hash, Hasher, SipHasher};
-use std::mem;
-use std::path::PathBuf;
 
-use url::{self, Url};
+use url::Url;
 
 use core::source::{Source, SourceId};
 use core::GitReference;
 use core::{Package, PackageId, Summary, Registry, Dependency};
-use util::{CargoResult, Config, to_hex};
+use util::{CargoResult, Config, FileLock, to_hex};
 use sources::PathSource;
 use sources::git::utils::{GitRemote, GitRevision};
 
@@ -17,11 +15,11 @@ use sources::git::utils::{GitRemote, GitRevision};
 pub struct GitSource<'cfg> {
     remote: GitRemote,
     reference: GitReference,
-    db_path: PathBuf,
-    checkout_path: PathBuf,
     source_id: SourceId,
     path_source: Option<PathSource<'cfg>>,
     rev: Option<GitRevision>,
+    checkout_lock: Option<FileLock>,
+    ident: String,
     config: &'cfg Config,
 }
 
@@ -30,25 +28,9 @@ impl<'cfg> GitSource<'cfg> {
                config: &'cfg Config) -> GitSource<'cfg> {
         assert!(source_id.is_git(), "id is not git, id={}", source_id);
 
-        let reference = match source_id.git_reference() {
-            Some(reference) => reference,
-            None => panic!("Not a git source; id={}", source_id),
-        };
-
         let remote = GitRemote::new(source_id.url());
         let ident = ident(source_id.url());
 
-        let db_path = config.git_db_path().join(&ident);
-
-        let reference_path = match *reference {
-            GitReference::Branch(ref s) |
-            GitReference::Tag(ref s) |
-            GitReference::Rev(ref s) => s.to_string(),
-        };
-        let checkout_path = config.git_checkout_path()
-                                  .join(&ident)
-                                  .join(&reference_path);
-
         let reference = match source_id.precise() {
             Some(s) => GitReference::Rev(s.to_string()),
             None => source_id.git_reference().unwrap().clone(),
@@ -57,11 +39,11 @@ impl<'cfg> GitSource<'cfg> {
         GitSource {
             remote: remote,
             reference: reference,
-            db_path: db_path,
-            checkout_path: checkout_path,
             source_id: source_id.clone(),
             path_source: None,
             rev: None,
+            checkout_lock: None,
+            ident: ident,
             config: config,
         }
     }
@@ -79,15 +61,11 @@ impl<'cfg> GitSource<'cfg> {
 fn ident(url: &Url) -> String {
     let mut hasher = SipHasher::new_with_keys(0,0);
 
-    // FIXME: this really should be able to not use to_str() everywhere, but the
-    //        compiler seems to currently ask for static lifetimes spuriously.
-    //        Perhaps related to rust-lang/rust#15144
     let url = canonicalize_url(url);
-    let ident = url.path().unwrap_or(&[])
-                   .last().map(|a| a.clone()).unwrap_or(String::new());
+    let ident = url.path_segments().and_then(|mut s| s.next_back()).unwrap_or("");
 
     let ident = if ident == "" {
-        "_empty".to_string()
+        "_empty"
     } else {
         ident
     };
@@ -101,39 +79,27 @@ pub fn canonicalize_url(url: &Url) -> Url {
     let mut url = url.clone();
 
     // Strip a trailing slash
-    if let url::SchemeData::Relative(ref mut rel) = url.scheme_data {
-        if rel.path.last().map(|s| s.is_empty()).unwrap_or(false) {
-            rel.path.pop();
-        }
-    }
+    url.path_segments_mut().unwrap().pop_if_empty();
 
     // HACKHACK: For github URL's specifically just lowercase
     // everything.  GitHub treats both the same, but they hash
     // differently, and we're gonna be hashing them. This wants a more
     // general solution, and also we're almost certainly not using the
     // same case conversion rules that GitHub does. (#84)
-    if url.domain() == Some("github.com") {
-        url.scheme = "https".to_string();
-        if let url::SchemeData::Relative(ref mut rel) = url.scheme_data {
-            rel.port = Some(443);
-            rel.default_port = Some(443);
-            let path = mem::replace(&mut rel.path, Vec::new());
-            rel.path = path.into_iter().map(|s| {
-                s.chars().flat_map(|c| c.to_lowercase()).collect()
-            }).collect();
-        }
+    if url.host_str() == Some("github.com") {
+        url.set_scheme("https").unwrap();
+        let path = url.path().to_lowercase();
+        url.set_path(&path);
     }
 
     // Repos generally can be accessed with or w/o '.git'
-    if let url::SchemeData::Relative(ref mut rel) = url.scheme_data {
-        let needs_chopping = {
-            let last = rel.path.last().map(|s| &s[..]).unwrap_or("");
-            last.ends_with(".git")
+    let needs_chopping = url.path().ends_with(".git");
+    if needs_chopping {
+        let last = {
+            let last = url.path_segments().unwrap().next_back().unwrap();
+            last[..last.len() - 4].to_owned()
         };
-        if needs_chopping {
-            let last = rel.path.pop().unwrap();
-            rel.path.push(last[..last.len() - 4].to_string())
-        }
+        url.path_segments_mut().unwrap().pop().push(&last);
     }
 
     url
@@ -160,7 +126,34 @@ impl<'cfg> Registry for GitSource<'cfg> {
 
 impl<'cfg> Source for GitSource<'cfg> {
     fn update(&mut self) -> CargoResult<()> {
-        let actual_rev = self.remote.rev_for(&self.db_path, &self.reference);
+        // First, lock both the global database and checkout locations that
+        // we're going to use. We may be performing a fetch into these locations
+        // so we need writable access.
+        let db_lock = format!(".cargo-lock-{}", self.ident);
+        let db_lock = try!(self.config.git_db_path()
+                                      .open_rw(&db_lock, self.config,
+                                               "the git database"));
+        let db_path = db_lock.parent().join(&self.ident);
+
+        let reference_path = match self.source_id.git_reference() {
+            Some(&GitReference::Branch(ref s)) |
+            Some(&GitReference::Tag(ref s)) |
+            Some(&GitReference::Rev(ref s)) => s,
+            None => panic!("not a git source"),
+        };
+        let checkout_lock = format!(".cargo-lock-{}-{}", self.ident,
+                                    reference_path);
+        let checkout_lock = try!(self.config.git_checkout_path()
+                                     .join(&self.ident)
+                                     .open_rw(&checkout_lock, self.config,
+                                              "the git checkout"));
+        let checkout_path = checkout_lock.parent().join(reference_path);
+
+        // Resolve our reference to an actual revision, and check if the
+        // databaes already has that revision. If it does, we just load a
+        // database pinned at that revision, and if we don't we issue an update
+        // to try to find the revision.
+        let actual_rev = self.remote.rev_for(&db_path, &self.reference);
         let should_update = actual_rev.is_err() ||
                             self.source_id.precise().is_none();
 
@@ -169,21 +162,30 @@ impl<'cfg> Source for GitSource<'cfg> {
                 format!("git repository `{}`", self.remote.url())));
 
             trace!("updating git source `{:?}`", self.remote);
-            let repo = try!(self.remote.checkout(&self.db_path));
+
+            let repo = try!(self.remote.checkout(&db_path, &self.config));
             let rev = try!(repo.rev_for(&self.reference));
             (repo, rev)
         } else {
-            (try!(self.remote.db_at(&self.db_path)), actual_rev.unwrap())
+            (try!(self.remote.db_at(&db_path)), actual_rev.unwrap())
         };
 
-        try!(repo.copy_to(actual_rev.clone(), &self.checkout_path));
+        // Copy the database to the checkout location. After this we could drop
+        // the lock on the database as we no longer needed it, but we leave it
+        // in scope so the destructors here won't tamper with too much.
+        try!(repo.copy_to(actual_rev.clone(), &checkout_path, &self.config));
 
         let source_id = self.source_id.with_precise(Some(actual_rev.to_string()));
-        let path_source = PathSource::new(&self.checkout_path, &source_id,
-                                          self.config);
+        let path_source = PathSource::new_recursive(&checkout_path,
+                                                    &source_id,
+                                                    self.config);
 
+        // Cache the information we just learned, and crucially also cache the
+        // lock on the checkout location. We wouldn't want someone else to come
+        // swipe our checkout location to another revision while we're using it!
         self.path_source = Some(path_source);
         self.rev = Some(actual_rev);
+        self.checkout_lock = Some(checkout_lock);
         self.path_source.as_mut().unwrap().update()
     }
 
