@@ -30,15 +30,17 @@ _cargo()
 	local opt__generate_lockfile="${opt__fetch}"
 	local opt__git_checkout="$opt_common --reference --url"
 	local opt__help="$opt_help"
+	local opt__init="$opt_common --bin --name --vcs"
 	local opt__install="$opt_common $opt_feat $opt_jobs --bin --branch --debug --example --git --list --path --rev --root --tag --vers"
 	local opt__locate_project="$opt_mani -h --help"
 	local opt__login="$opt_common --host"
+	local opt__metadata="$opt_common $opt_feat $opt_mani --format-version"
 	local opt__new="$opt_common --vcs --bin --name"
 	local opt__owner="$opt_common -a --add -r --remove -l --list --index --token"
 	local opt__package="$opt_common $opt_mani -l --list --no-verify --no-metadata"
 	local opt__pkgid="${opt__fetch}"
 	local opt__publish="$opt_common $opt_mani --host --token --no-verify"
-	local opt__read_manifest="$opt_help $opt_verbose $opt_mani --color"
+	local opt__read_manifest="$opt_help $opt_verbose $opt_mani $opt_color"
 	local opt__run="$opt_common $opt_feat $opt_mani $opt_jobs --target --bin --example --release"
 	local opt__rustc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_jobs --target --lib --bin --test --bench --example --release"
 	local opt__rustdoc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_jobs --target --lib --bin --test --bench --example --release --open"
@@ -47,7 +49,7 @@ _cargo()
 	local opt__uninstall="$opt_common --bin --root"
 	local opt__update="$opt_common $opt_pkg $opt_mani --aggressive --precise"
 	local opt__verify_project="${opt__fetch}"
-	local opt__version="$opt_help $opt_verbose --color"
+	local opt__version="$opt_help $opt_verbose $opt_color"
 	local opt__yank="$opt_common --vers --undo --index --token"
 
 	if [[ $cword -eq 1 ]]; then
