@@ -4,7 +4,6 @@ use std::path::Path;
 use git2;
 
 use support::{git, project, execs, main_file, path2url};
-use support::{COMPILING, UPDATING, RUNNING};
 use support::paths::{self, CargoPathExt};
 use hamcrest::{assert_that,existing_file};
 use cargo::util::process;
@@ -57,13 +56,12 @@ test!(cargo_compile_simple_git_dep {
 
     assert_that(project.cargo_process("build"),
         execs()
-        .with_stdout(&format!("{} git repository `{}`\n\
-                              {} dep1 v0.5.0 ({}#[..])\n\
-                              {} foo v0.5.0 ({})\n",
-                             UPDATING, path2url(git_root.clone()),
-                             COMPILING, path2url(git_root),
-                             COMPILING, path2url(root)))
-        .with_stderr(""));
+        .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                              [COMPILING] dep1 v0.5.0 ({}#[..])\n\
+                              [COMPILING] foo v0.5.0 ({})\n",
+                             path2url(git_root.clone()),
+                             path2url(git_root),
+                             path2url(root))));
 
     assert_that(&project.bin("foo"), existing_file());
 
@@ -124,13 +122,12 @@ test!(cargo_compile_git_dep_branch {
 
     assert_that(project.cargo_process("build"),
         execs()
-        .with_stdout(&format!("{} git repository `{}`\n\
-                              {} dep1 v0.5.0 ({}?branch=branchy#[..])\n\
-                              {} foo v0.5.0 ({})\n",
-                             UPDATING, path2url(git_root.clone()),
-                             COMPILING, path2url(git_root),
-                             COMPILING, path2url(root)))
-        .with_stderr(""));
+        .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                              [COMPILING] dep1 v0.5.0 ({}?branch=branchy#[..])\n\
+                              [COMPILING] foo v0.5.0 ({})\n",
+                             path2url(git_root.clone()),
+                             path2url(git_root),
+                             path2url(root))));
 
     assert_that(&project.bin("foo"), existing_file());
 
@@ -194,12 +191,12 @@ test!(cargo_compile_git_dep_tag {
 
     assert_that(project.cargo_process("build"),
         execs()
-        .with_stdout(&format!("{} git repository `{}`\n\
-                              {} dep1 v0.5.0 ({}?tag=v0.1.0#[..])\n\
-                              {} foo v0.5.0 ({})\n",
-                             UPDATING, path2url(git_root.clone()),
-                             COMPILING, path2url(git_root),
-                             COMPILING, path2url(root))));
+        .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                              [COMPILING] dep1 v0.5.0 ({}?tag=v0.1.0#[..])\n\
+                              [COMPILING] foo v0.5.0 ({})\n",
+                             path2url(git_root.clone()),
+                             path2url(git_root),
+                             path2url(root))));
 
     assert_that(&project.bin("foo"), existing_file());
 
@@ -381,7 +378,7 @@ test!(cargo_compile_with_short_ssh_git {
         execs()
         .with_stdout("")
         .with_stderr(&format!("\
-failed to parse manifest at `[..]`
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   invalid url `{}`: relative URL without a base
@@ -497,12 +494,12 @@ test!(recompilation {
 
     // First time around we should compile both foo and bar
     assert_that(p.cargo_process("build"),
-                execs().with_stdout(&format!("{} git repository `{}`\n\
-                                             {} bar v0.5.0 ({}#[..])\n\
-                                             {} foo v0.5.0 ({})\n",
-                                            UPDATING, git_project.url(),
-                                            COMPILING, git_project.url(),
-                                            COMPILING, p.url())));
+                execs().with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                                             [COMPILING] bar v0.5.0 ({}#[..])\n\
+                                             [COMPILING] foo v0.5.0 ({})\n",
+                                            git_project.url(),
+                                            git_project.url(),
+                                            p.url())));
 
     // Don't recompile the second time
     assert_that(p.cargo("build"),
@@ -517,8 +514,7 @@ test!(recompilation {
                 execs().with_stdout(""));
 
     assert_that(p.cargo("update"),
-                execs().with_stdout(&format!("{} git repository `{}`",
-                                            UPDATING,
+                execs().with_stderr(&format!("[UPDATING] git repository `{}`",
                                             git_project.url())));
 
     assert_that(p.cargo("build"),
@@ -537,26 +533,24 @@ test!(recompilation {
 
     // Update the dependency and carry on!
     assert_that(p.cargo("update"),
-                execs().with_stdout(&format!("{} git repository `{}`\n\
-                                              {} bar v0.5.0 ([..]) -> #[..]\n\
+                execs().with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                                              [UPDATING] bar v0.5.0 ([..]) -> #[..]\n\
                                              ",
-                                            UPDATING,
-                                            git_project.url(),
-                                            UPDATING)));
+                                            git_project.url())));
     println!("going for the last compile");
     assert_that(p.cargo("build"),
-                execs().with_stdout(&format!("{} bar v0.5.0 ({}#[..])\n\
-                                             {} foo v0.5.0 ({})\n",
-                                            COMPILING, git_project.url(),
-                                            COMPILING, p.url())));
+                execs().with_stderr(&format!("[COMPILING] bar v0.5.0 ({}#[..])\n\
+                                             [COMPILING] foo v0.5.0 ({})\n",
+                                            git_project.url(),
+                                            p.url())));
 
     // Make sure clean only cleans one dep
     assert_that(p.cargo("clean")
                  .arg("-p").arg("foo"),
                 execs().with_stdout(""));
     assert_that(p.cargo("build"),
-                execs().with_stdout(&format!("{} foo v0.5.0 ({})\n",
-                                            COMPILING, p.url())));
+                execs().with_stderr(&format!("[COMPILING] foo v0.5.0 ({})\n",
+                                            p.url())));
 });
 
 test!(update_with_shared_deps {
@@ -619,14 +613,12 @@ test!(update_with_shared_deps {
 
     // First time around we should compile both foo and bar
     assert_that(p.cargo_process("build"),
-                execs().with_stdout(&format!("\
-{updating} git repository `{git}`
-{compiling} bar v0.5.0 ({git}#[..])
-{compiling} [..] v0.5.0 ({dir})
-{compiling} [..] v0.5.0 ({dir})
-{compiling} foo v0.5.0 ({dir})\n",
-                    updating = UPDATING, git = git_project.url(),
-                    compiling = COMPILING, dir = p.url())));
+                execs().with_stderr(&format!("\
+[UPDATING] git repository `{git}`
+[COMPILING] bar v0.5.0 ({git}#[..])
+[COMPILING] [..] v0.5.0 ([..])
+[COMPILING] [..] v0.5.0 ([..])
+[COMPILING] foo v0.5.0 ({dir})\n", git = git_project.url(), dir = p.url())));
 
     // Modify a file manually, and commit it
     File::create(&git_project.root().join("src/bar.rs")).unwrap().write_all(br#"
@@ -651,7 +643,8 @@ test!(update_with_shared_deps {
                  .arg("-p").arg("bar")
                  .arg("--precise").arg("0.1.2"),
                 execs().with_status(101).with_stderr("\
-Unable to update [..]
+[UPDATING] git repository [..]
+[ERROR] Unable to update [..]
 
 To learn more, run the command again with --verbose.
 "));
@@ -669,28 +662,23 @@ To learn more, run the command again with --verbose.
     assert_that(p.cargo("update")
                  .arg("-p").arg("dep1")
                  .arg("--aggressive"),
-                execs().with_stdout(&format!("{} git repository `{}`\n\
-                                              {} bar v0.5.0 ([..]) -> #[..]\n\
-                                             ",
-                                            UPDATING,
-                                            git_project.url(),
-                                            UPDATING)));
+                execs().with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                                              [UPDATING] bar v0.5.0 ([..]) -> #[..]\n\
+                                             ", git_project.url())));
 
     // Make sure we still only compile one version of the git repo
     println!("build");
     assert_that(p.cargo("build"),
-                execs().with_stdout(&format!("\
-{compiling} bar v0.5.0 ({git}#[..])
-{compiling} [..] v0.5.0 ({dir})
-{compiling} [..] v0.5.0 ({dir})
-{compiling} foo v0.5.0 ({dir})\n",
-                    git = git_project.url(),
-                    compiling = COMPILING, dir = p.url())));
+                execs().with_stderr(&format!("\
+[COMPILING] bar v0.5.0 ({git}#[..])
+[COMPILING] [..] v0.5.0 ({dir}[..]dep[..])
+[COMPILING] [..] v0.5.0 ({dir}[..]dep[..])
+[COMPILING] foo v0.5.0 ({dir})\n",
+                    git = git_project.url(), dir = p.url())));
 
     // We should be able to update transitive deps
     assert_that(p.cargo("update").arg("-p").arg("bar"),
-                execs().with_stdout(&format!("{} git repository `{}`",
-                                            UPDATING,
+                execs().with_stderr(&format!("[UPDATING] git repository `{}`",
                                             git_project.url())));
 });
 
@@ -732,7 +720,10 @@ test!(dep_with_submodule {
         ");
 
     assert_that(project.cargo_process("build"),
-                execs().with_stderr("").with_status(0));
+                execs().with_stderr("\
+[UPDATING] git repository [..]
+[COMPILING] dep1 [..]
+[COMPILING] foo [..]").with_status(0));
 });
 
 test!(two_deps_only_update_one {
@@ -775,17 +766,12 @@ test!(two_deps_only_update_one {
 
     assert_that(project.cargo_process("build"),
         execs()
-        .with_stdout(&format!("{} git repository `[..]`\n\
-                              {} git repository `[..]`\n\
-                              {} [..] v0.5.0 ([..])\n\
-                              {} [..] v0.5.0 ([..])\n\
-                              {} foo v0.5.0 ({})\n",
-                             UPDATING,
-                             UPDATING,
-                             COMPILING,
-                             COMPILING,
-                             COMPILING, project.url()))
-        .with_stderr(""));
+        .with_stderr(&format!("[UPDATING] git repository `[..]`\n\
+                              [UPDATING] git repository `[..]`\n\
+                              [COMPILING] [..] v0.5.0 ([..])\n\
+                              [COMPILING] [..] v0.5.0 ([..])\n\
+                              [COMPILING] foo v0.5.0 ({})\n",
+                             project.url())));
 
     File::create(&git1.root().join("src/lib.rs")).unwrap().write_all(br#"
         pub fn foo() {}
@@ -797,13 +783,9 @@ test!(two_deps_only_update_one {
     assert_that(project.cargo("update")
                        .arg("-p").arg("dep1"),
         execs()
-        .with_stdout(&format!("{} git repository `{}`\n\
-                               {} dep1 v0.5.0 ([..]) -> #[..]\n\
-                              ",
-                             UPDATING,
-                             git1.url(),
-                             UPDATING))
-        .with_stderr(""));
+        .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                               [UPDATING] dep1 v0.5.0 ([..]) -> #[..]\n\
+                              ", git1.url())));
 });
 
 test!(stale_cached_version {
@@ -868,11 +850,11 @@ test!(stale_cached_version {
     // Now build!
     assert_that(foo.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{updating} git repository `{bar}`
-{compiling} bar v0.0.0 ({bar}#[..])
-{compiling} foo v0.0.0 ({foo})
-", updating = UPDATING, compiling = COMPILING, bar = bar.url(), foo = foo.url())));
+                       .with_stderr(&format!("\
+[UPDATING] git repository `{bar}`
+[COMPILING] bar v0.0.0 ({bar}#[..])
+[COMPILING] foo v0.0.0 ({foo})
+", bar = bar.url(), foo = foo.url())));
     assert_that(foo.process(&foo.bin("foo")), execs().with_status(0));
 });
 
@@ -919,17 +901,11 @@ test!(dep_with_changed_submodule {
 
     println!("first run");
     assert_that(project.cargo_process("run"), execs()
-                .with_stdout(&format!("{} git repository `[..]`\n\
-                                      {} dep1 v0.5.0 ([..])\n\
-                                      {} foo v0.5.0 ([..])\n\
-                                      {} `target[..]foo[..]`\n\
-                                      project2\
-                                      ",
-                                      UPDATING,
-                                      COMPILING,
-                                      COMPILING,
-                                      RUNNING))
-                .with_stderr("")
+                .with_stderr("[UPDATING] git repository `[..]`\n\
+                                      [COMPILING] dep1 v0.5.0 ([..])\n\
+                                      [COMPILING] foo v0.5.0 ([..])\n\
+                                      [RUNNING] `target[..]foo[..]`\n")
+                .with_stdout("project2\n")
                 .with_status(0));
 
     File::create(&git_project.root().join(".gitmodules")).unwrap()
@@ -960,22 +936,16 @@ test!(dep_with_changed_submodule {
     assert_that(project.cargo("update").arg("-v"),
                 execs()
                 .with_stderr("")
-                .with_stdout(&format!("{} git repository `{}`\n\
-                                       {} dep1 v0.5.0 ([..]) -> #[..]\n\
-                                      ",
-                                     UPDATING,
-                                     git_project.url(),
-                                     UPDATING)));
+                .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                                       [UPDATING] dep1 v0.5.0 ([..]) -> #[..]\n\
+                                      ", git_project.url())));
 
     println!("last run");
     assert_that(project.cargo("run"), execs()
-                .with_stdout(&format!("{compiling} dep1 v0.5.0 ([..])\n\
-                                      {compiling} foo v0.5.0 ([..])\n\
-                                      {running} `target[..]foo[..]`\n\
-                                      project3\
-                                      ",
-                                      compiling = COMPILING, running = RUNNING))
-                .with_stderr("")
+                .with_stderr("[COMPILING] dep1 v0.5.0 ([..])\n\
+                                      [COMPILING] foo v0.5.0 ([..])\n\
+                                      [RUNNING] `target[..]foo[..]`\n")
+                .with_stdout("project3\n")
                 .with_status(0));
 });
 
@@ -1017,25 +987,25 @@ test!(dev_deps_with_testing {
     // Generate a lockfile which did not use `bar` to compile, but had to update
     // `bar` to generate the lockfile
     assert_that(p.cargo_process("build"),
-        execs().with_stdout(&format!("\
-{updating} git repository `{bar}`
-{compiling} foo v0.5.0 ({url})
-", updating = UPDATING, compiling = COMPILING, url = p.url(), bar = p2.url())));
+        execs().with_stderr(&format!("\
+[UPDATING] git repository `{bar}`
+[COMPILING] foo v0.5.0 ({url})
+", url = p.url(), bar = p2.url())));
 
     // Make sure we use the previous resolution of `bar` instead of updating it
     // a second time.
     assert_that(p.cargo("test"),
-        execs().with_stdout(&format!("\
-{compiling} [..] v0.5.0 ([..])
-{compiling} [..] v0.5.0 ([..]
-{running} target[..]foo-[..]
-
+                execs().with_stderr("\
+[COMPILING] [..] v0.5.0 ([..])
+[COMPILING] [..] v0.5.0 ([..]
+[RUNNING] target[..]foo-[..]")
+                       .with_stdout("
 running 1 test
 test tests::foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING)));
+"));
 });
 
 test!(git_build_cmd_freshness {
@@ -1059,9 +1029,9 @@ test!(git_build_cmd_freshness {
 
     assert_that(foo.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.0 ({url})
-", compiling = COMPILING, url = foo.url())));
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.0 ({url})
+", url = foo.url())));
 
     // Smoke test to make sure it doesn't compile again
     println!("first pass");
@@ -1110,10 +1080,10 @@ test!(git_name_not_always_needed {
     // Generate a lockfile which did not use `bar` to compile, but had to update
     // `bar` to generate the lockfile
     assert_that(p.cargo_process("build"),
-        execs().with_stdout(&format!("\
-{updating} git repository `{bar}`
-{compiling} foo v0.5.0 ({url})
-", updating = UPDATING, compiling = COMPILING, url = p.url(), bar = p2.url())));
+        execs().with_stderr(&format!("\
+[UPDATING] git repository `{bar}`
+[COMPILING] foo v0.5.0 ({url})
+", url = p.url(), bar = p2.url())));
 });
 
 test!(git_repo_changing_no_rebuild {
@@ -1143,11 +1113,11 @@ test!(git_repo_changing_no_rebuild {
     p1.build();
     p1.root().move_into_the_past().unwrap();
     assert_that(p1.cargo("build"),
-                execs().with_stdout(&format!("\
-{updating} git repository `{bar}`
-{compiling} [..]
-{compiling} [..]
-", updating = UPDATING, compiling = COMPILING, bar = bar.url())));
+                execs().with_stderr(&format!("\
+[UPDATING] git repository `{bar}`
+[COMPILING] [..]
+[COMPILING] [..]
+", bar = bar.url())));
 
     // Make a commit to lock p2 to a different rev
     File::create(&bar.root().join("src/lib.rs")).unwrap().write_all(br#"
@@ -1169,11 +1139,11 @@ test!(git_repo_changing_no_rebuild {
         "#, bar.url()))
         .file("src/main.rs", "fn main() {}");
     assert_that(p2.cargo_process("build"),
-                execs().with_stdout(&format!("\
-{updating} git repository `{bar}`
-{compiling} [..]
-{compiling} [..]
-", updating = UPDATING, compiling = COMPILING, bar = bar.url())));
+                execs().with_stderr(&format!("\
+[UPDATING] git repository `{bar}`
+[COMPILING] [..]
+[COMPILING] [..]
+", bar = bar.url())));
 
     // And now for the real test! Make sure that p1 doesn't get rebuilt
     // even though the git repo has changed.
@@ -1265,9 +1235,9 @@ test!(fetch_downloads {
         "#, bar.url()))
         .file("src/main.rs", "fn main() {}");
     assert_that(p.cargo_process("fetch"),
-                execs().with_status(0).with_stdout(&format!("\
-{updating} git repository `{url}`
-", updating = UPDATING, url = bar.url())));
+                execs().with_status(0).with_stderr(&format!("\
+[UPDATING] git repository `{url}`
+", url = bar.url())));
 
     assert_that(p.cargo("fetch"),
                 execs().with_status(0).with_stdout(""));
@@ -1297,13 +1267,12 @@ test!(warnings_in_git_dep {
 
     assert_that(p.cargo_process("build"),
         execs()
-        .with_stdout(&format!("{} git repository `{}`\n\
-                              {} bar v0.5.0 ({}#[..])\n\
-                              {} foo v0.5.0 ({})\n",
-                             UPDATING, bar.url(),
-                             COMPILING, bar.url(),
-                             COMPILING, p.url()))
-        .with_stderr(""));
+        .with_stderr(&format!("[UPDATING] git repository `{}`\n\
+                              [COMPILING] bar v0.5.0 ({}#[..])\n\
+                              [COMPILING] foo v0.5.0 ({})\n",
+                             bar.url(),
+                             bar.url(),
+                             p.url())));
 });
 
 test!(update_ambiguous {
@@ -1356,7 +1325,7 @@ test!(update_ambiguous {
                  .arg("-p").arg("foo"),
                 execs().with_status(101)
                        .with_stderr("\
-There are multiple `foo` packages in your project, and the specification `foo` \
+[ERROR] There are multiple `foo` packages in your project, and the specification `foo` \
 is ambiguous.
 Please re-run this command with `-p <spec>` where `<spec>` is one of the \
 following:
@@ -1400,8 +1369,8 @@ test!(update_one_dep_in_repo_with_many_deps {
     assert_that(p.cargo("update")
                  .arg("-p").arg("foo"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-Updating git repository `{}`
+                       .with_stderr(&format!("\
+[UPDATING] git repository `{}`
 ", foo.url())));
 });
 
@@ -1454,13 +1423,13 @@ test!(switch_deps_does_not_update_transitive {
     p.build();
     assert_that(p.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-Updating git repository `{}`
-Updating git repository `{}`
-{compiling} transitive [..]
-{compiling} dep [..]
-{compiling} project [..]
-", dep1.url(), transitive.url(), compiling = COMPILING)));
+                       .with_stderr(&format!("\
+[UPDATING] git repository `{}`
+[UPDATING] git repository `{}`
+[COMPILING] transitive [..]
+[COMPILING] dep [..]
+[COMPILING] project [..]
+", dep1.url(), transitive.url())));
 
     // Update the dependency to point to the second repository, but this
     // shouldn't update the transitive dependency which is the same.
@@ -1475,11 +1444,11 @@ Updating git repository `{}`
 
     assert_that(p.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-Updating git repository `{}`
-{compiling} dep [..]
-{compiling} project [..]
-", dep2.url(), compiling = COMPILING)));
+                       .with_stderr(&format!("\
+[UPDATING] git repository `{}`
+[COMPILING] dep [..]
+[COMPILING] project [..]
+", dep2.url())));
 });
 
 test!(update_one_source_updates_all_packages_in_that_git_source {
@@ -1575,17 +1544,17 @@ test!(switch_sources {
             [dependencies.a]
             git = '{}'
         "#, a1.url()))
-        .file("b/src/lib.rs", "fn main() {}");
+        .file("b/src/lib.rs", "pub fn main() {}");
 
     p.build();
     assert_that(p.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{updating} git repository `file://[..]a1`
-{compiling} a v0.5.0 ([..]a1#[..]
-{compiling} b v0.5.0 ([..])
-{compiling} project v0.5.0 ([..])
-", updating = UPDATING, compiling = COMPILING)));
+                       .with_stderr("\
+[UPDATING] git repository `file://[..]a1`
+[COMPILING] a v0.5.0 ([..]a1#[..]
+[COMPILING] b v0.5.0 ([..])
+[COMPILING] project v0.5.0 ([..])
+"));
 
     File::create(&p.root().join("b/Cargo.toml")).unwrap().write_all(format!(r#"
         [project]
@@ -1598,12 +1567,12 @@ test!(switch_sources {
 
     assert_that(p.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{updating} git repository `file://[..]a2`
-{compiling} a v0.5.1 ([..]a2#[..]
-{compiling} b v0.5.0 ([..])
-{compiling} project v0.5.0 ([..])
-", updating = UPDATING, compiling = COMPILING)));
+                       .with_stderr("\
+[UPDATING] git repository `file://[..]a2`
+[COMPILING] a v0.5.1 ([..]a2#[..]
+[COMPILING] b v0.5.0 ([..])
+[COMPILING] project v0.5.0 ([..])
+"));
 });
 
 test!(dont_require_submodules_are_checked_out {
@@ -1704,11 +1673,11 @@ test!(lints_are_suppressed {
         .file("src/lib.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(&format!("\
-{updating} git repository `[..]`
-{compiling} a v0.5.0 ([..])
-{compiling} foo v0.0.1 ([..])
-", compiling = COMPILING, updating = UPDATING)));
+                execs().with_status(0).with_stderr("\
+[UPDATING] git repository `[..]`
+[COMPILING] a v0.5.0 ([..])
+[COMPILING] foo v0.0.1 ([..])
+"));
 });
 
 test!(denied_lints_are_allowed {
@@ -1741,9 +1710,55 @@ test!(denied_lints_are_allowed {
         .file("src/lib.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(&format!("\
-{updating} git repository `[..]`
-{compiling} a v0.5.0 ([..])
-{compiling} foo v0.0.1 ([..])
-", compiling = COMPILING, updating = UPDATING)));
+                execs().with_status(0).with_stderr("\
+[UPDATING] git repository `[..]`
+[COMPILING] a v0.5.0 ([..])
+[COMPILING] foo v0.0.1 ([..])
+"));
+});
+
+test!(add_a_git_dep {
+    let git = git::new("git", |p| {
+        p.file("Cargo.toml", r#"
+            [project]
+            name = "git"
+            version = "0.5.0"
+            authors = []
+        "#)
+        .file("src/lib.rs", "")
+    }).unwrap();
+
+    let p = project("foo")
+        .file("Cargo.toml", &format!(r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies]
+            a = {{ path = 'a' }}
+            git = {{ git = '{}' }}
+        "#, git.url()))
+        .file("src/lib.rs", "")
+        .file("a/Cargo.toml", r#"
+            [package]
+            name = "a"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("a/src/lib.rs", "");
+
+    assert_that(p.cargo_process("build"), execs().with_status(0));
+
+    File::create(p.root().join("a/Cargo.toml")).unwrap().write_all(format!(r#"
+        [package]
+        name = "a"
+        version = "0.0.1"
+        authors = []
+
+        [dependencies]
+        git = {{ git = '{}' }}
+    "#, git.url()).as_bytes()).unwrap();
+
+    assert_that(p.cargo("build"), execs().with_status(0));
 });
