@@ -2,7 +2,6 @@ use std::fs;
 use std::env;
 
 use support::{project, execs};
-use support::{COMPILING, RUNNING};
 use hamcrest::assert_that;
 
 fn setup() {
@@ -263,8 +262,9 @@ test!(native_plugin_dependency_with_custom_ar_linker {
 
     foo.build();
     assert_that(bar.cargo_process("build").arg("--verbose"),
-                execs().with_stdout(&format!("\
-{compiling} foo v0.0.1 ({url})
-{running} `rustc [..] -C ar=nonexistent-ar -C linker=nonexistent-linker [..]`
-", compiling = COMPILING, running = RUNNING, url = bar.url())))
+                execs().with_stderr_contains("\
+[COMPILING] foo v0.0.1 ([..])
+[RUNNING] `rustc [..] -C ar=nonexistent-ar -C linker=nonexistent-linker [..]`
+[ERROR] could not exec the linker [..]
+"));
 });
