@@ -2,7 +2,6 @@ use std::fs::File;
 use std::io::prelude::*;
 
 use support::{project, execs};
-use support::{COMPILING, FRESH};
 use support::paths::CargoPathExt;
 use hamcrest::assert_that;
 
@@ -23,12 +22,12 @@ test!(invalid1 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` includes `baz` which is neither a dependency nor another feature
-")));
+"));
 });
 
 test!(invalid2 {
@@ -48,12 +47,12 @@ test!(invalid2 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Features and dependencies cannot have the same name: `bar`
-")));
+"));
 });
 
 test!(invalid3 {
@@ -73,13 +72,13 @@ test!(invalid3 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` depends on `baz` which is not an optional dependency.
 Consider adding `optional = true` to the dependency
-")));
+"));
 });
 
 test!(invalid4 {
@@ -104,9 +103,9 @@ test!(invalid4 {
         .file("bar/src/lib.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(101).with_stderr(&format!("\
-Package `bar v0.0.1 ([..])` does not have these features: `bar`
-")));
+                execs().with_status(101).with_stderr("\
+[ERROR] Package `bar v0.0.1 ([..])` does not have these features: `bar`
+"));
 
     let p = p.file("Cargo.toml", r#"
             [project]
@@ -116,9 +115,9 @@ Package `bar v0.0.1 ([..])` does not have these features: `bar`
         "#);
 
     assert_that(p.cargo_process("build").arg("--features").arg("test"),
-                execs().with_status(101).with_stderr(&format!("\
-Package `foo v0.0.1 ([..])` does not have these features: `test`
-")));
+                execs().with_status(101).with_stderr("\
+[ERROR] Package `foo v0.0.1 ([..])` does not have these features: `test`
+"));
 });
 
 test!(invalid5 {
@@ -136,12 +135,12 @@ test!(invalid5 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Dev-dependencies are not allowed to be optional: `bar`
-")));
+"));
 });
 
 test!(invalid6 {
@@ -158,12 +157,12 @@ test!(invalid6 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build").arg("--features").arg("foo"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `foo` requires `bar` which is not an optional dependency
-")));
+"));
 });
 
 test!(invalid7 {
@@ -181,12 +180,12 @@ test!(invalid7 {
         .file("src/main.rs", "");
 
     assert_that(p.cargo_process("build").arg("--features").arg("foo"),
-                execs().with_status(101).with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                execs().with_status(101).with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `foo` requires `bar` which is not an optional dependency
-")));
+"));
 });
 
 test!(invalid8 {
@@ -211,9 +210,9 @@ test!(invalid8 {
         .file("bar/src/lib.rs", "");
 
     assert_that(p.cargo_process("build").arg("--features").arg("foo"),
-                execs().with_status(101).with_stderr(&format!("\
-features in dependencies cannot enable features in other dependencies: `foo/bar`
-")));
+                execs().with_status(101).with_stderr("\
+[ERROR] features in dependencies cannot enable features in other dependencies: `foo/bar`
+"));
 });
 
 test!(no_feature_doesnt_build {
@@ -245,17 +244,17 @@ test!(no_feature_doesnt_build {
         .file("bar/src/lib.rs", "pub fn bar() {}");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
     assert_that(p.process(&p.bin("foo")),
                 execs().with_status(0).with_stdout(""));
 
     assert_that(p.cargo("build").arg("--features").arg("bar"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} bar v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] bar v0.0.1 ({dir}/bar)
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
     assert_that(p.process(&p.bin("foo")),
                 execs().with_status(0).with_stdout("bar\n"));
 });
@@ -292,17 +291,17 @@ test!(default_feature_pulled_in {
         .file("bar/src/lib.rs", "pub fn bar() {}");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} bar v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] bar v0.0.1 ({dir}/bar)
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
     assert_that(p.process(&p.bin("foo")),
                 execs().with_status(0).with_stdout("bar\n"));
 
     assert_that(p.cargo("build").arg("--no-default-features"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
     assert_that(p.process(&p.bin("foo")),
                 execs().with_status(0).with_stdout(""));
 });
@@ -322,7 +321,7 @@ test!(cyclic_feature {
 
     assert_that(p.cargo_process("build"),
                 execs().with_status(101).with_stderr("\
-Cyclic feature dependency: feature `default` depends on itself
+[ERROR] Cyclic feature dependency: feature `default` depends on itself
 "));
 });
 
@@ -342,7 +341,7 @@ test!(cyclic_feature2 {
 
     assert_that(p.cargo_process("build"),
                 execs().with_status(101).with_stderr("\
-Cyclic feature dependency: feature `[..]` depends on itself
+[ERROR] Cyclic feature dependency: feature `[..]` depends on itself
 "));
 });
 
@@ -393,11 +392,11 @@ test!(groups_on_groups_on_groups {
         .file("baz/src/lib.rs", "pub fn baz() {}");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} ba[..] v0.0.1 ({dir})
-{compiling} ba[..] v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
+[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
 });
 
 test!(many_cli_features {
@@ -437,11 +436,11 @@ test!(many_cli_features {
         .file("baz/src/lib.rs", "pub fn baz() {}");
 
     assert_that(p.cargo_process("build").arg("--features").arg("bar baz"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} ba[..] v0.0.1 ({dir})
-{compiling} ba[..] v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
+[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
 });
 
 test!(union_features {
@@ -498,11 +497,11 @@ test!(union_features {
         "#);
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} d2 v0.0.1 ({dir})
-{compiling} d1 v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] d2 v0.0.1 ({dir}/d2)
+[COMPILING] d1 v0.0.1 ({dir}/d1)
+[COMPILING] foo v0.0.1 ({dir})
+", dir = p.url())));
 });
 
 test!(many_features_no_rebuilds {
@@ -532,17 +531,17 @@ test!(many_features_no_rebuilds {
         .file("a/src/lib.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} a v0.1.0 ({dir})
-{compiling} b v0.1.0 ({dir})
-", compiling = COMPILING, dir = p.url())));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] a v0.1.0 ({dir}/a)
+[COMPILING] b v0.1.0 ({dir})
+", dir = p.url())));
     p.root().move_into_the_past().unwrap();
 
     assert_that(p.cargo("build").arg("-v"),
-                execs().with_status(0).with_stdout(format!("\
-{fresh} a v0.1.0 ([..])
-{fresh} b v0.1.0 ([..])
-", fresh = FRESH)));
+                execs().with_status(0).with_stderr("\
+[FRESH] a v0.1.0 ([..]/a)
+[FRESH] b v0.1.0 ([..])
+"));
 });
 
 // Tests that all cmd lines work with `--features ""`
@@ -765,9 +764,9 @@ test!(optional_and_dev_dep {
         .file("foo/src/lib.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} test v0.1.0 ([..])
-", compiling = COMPILING)));
+                execs().with_status(0).with_stderr("\
+[COMPILING] test v0.1.0 ([..])
+"));
 });
 
 test!(activating_feature_activates_dep {
