@@ -2,7 +2,6 @@ use std::fs::{self, File};
 use std::io::prelude::*;
 
 use support::{project, execs, path2url};
-use support::COMPILING;
 use support::paths::CargoPathExt;
 use hamcrest::{assert_that, existing_file};
 
@@ -22,9 +21,9 @@ test!(modifying_and_moving {
         .file("src/a.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = path2url(p.root()))));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = path2url(p.root()))));
 
     assert_that(p.cargo("build"),
                 execs().with_status(0).with_stdout(""));
@@ -32,11 +31,11 @@ test!(modifying_and_moving {
     p.root().join("target").move_into_the_past().unwrap();
 
     File::create(&p.root().join("src/a.rs")).unwrap()
-         .write_all(b"fn main() {}").unwrap();
+         .write_all(b"#[allow(unused)]fn main() {}").unwrap();
     assert_that(p.cargo("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = path2url(p.root()))));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = path2url(p.root()))));
 
     fs::rename(&p.root().join("src/a.rs"), &p.root().join("src/b.rs")).unwrap();
     assert_that(p.cargo("build"),
@@ -61,9 +60,9 @@ test!(modify_only_some_files {
         .file("tests/test.rs", "");
 
     assert_that(p.cargo_process("build"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = path2url(p.root()))));
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = path2url(p.root()))));
     assert_that(p.cargo("test"),
                 execs().with_status(0));
     ::sleep_ms(1000);
@@ -74,15 +73,14 @@ test!(modify_only_some_files {
     let bin = p.root().join("src/b.rs");
 
     File::create(&lib).unwrap().write_all(b"invalid rust code").unwrap();
-    File::create(&bin).unwrap().write_all(b"fn foo() {}").unwrap();
+    File::create(&bin).unwrap().write_all(b"#[allow(unused)]fn foo() {}").unwrap();
     lib.move_into_the_past().unwrap();
 
     // Make sure the binary is rebuilt, not the lib
-    assert_that(p.cargo("build")
-                 .env("RUST_LOG", "cargo::ops::cargo_rustc::fingerprint"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo v0.0.1 ({dir})
-", compiling = COMPILING, dir = path2url(p.root()))));
+    assert_that(p.cargo("build"),
+                execs().with_status(0).with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+", dir = path2url(p.root()))));
     assert_that(&p.bin("foo"), existing_file());
 });
 
@@ -152,19 +150,19 @@ test!(changing_features_is_ok {
 
     assert_that(p.cargo_process("build"),
                 execs().with_status(0)
-                       .with_stdout("\
+                       .with_stderr("\
 [..]Compiling foo v0.0.1 ([..])
 "));
 
     assert_that(p.cargo("build").arg("--features").arg("foo"),
                 execs().with_status(0)
-                       .with_stdout("\
+                       .with_stderr("\
 [..]Compiling foo v0.0.1 ([..])
 "));
 
     assert_that(p.cargo("build"),
                 execs().with_status(0)
-                       .with_stdout("\
+                       .with_stderr("\
 [..]Compiling foo v0.0.1 ([..])
 "));
 
@@ -250,11 +248,11 @@ test!(no_rebuild_transitive_target_deps {
                 execs().with_status(0));
     assert_that(p.cargo("test").arg("--no-run"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} c v0.0.1 ([..])
-{compiling} b v0.0.1 ([..])
-{compiling} foo v0.0.1 ([..])
-", compiling = COMPILING)));
+                       .with_stderr("\
+[COMPILING] c v0.0.1 ([..])
+[COMPILING] b v0.0.1 ([..])
+[COMPILING] foo v0.0.1 ([..])
+"));
 });
 
 test!(rerun_if_changed_in_dep {
@@ -288,3 +286,67 @@ test!(rerun_if_changed_in_dep {
     assert_that(p.cargo("build"),
                 execs().with_status(0).with_stdout(""));
 });
+
+test!(same_build_dir_cached_packages {
+    let p = project("foo")
+        .file("a1/Cargo.toml", r#"
+            [package]
+            name = "a1"
+            version = "0.0.1"
+            authors = []
+            [dependencies]
+            b = { path = "../b" }
+        "#)
+        .file("a1/src/lib.rs", "")
+        .file("a2/Cargo.toml", r#"
+            [package]
+            name = "a2"
+            version = "0.0.1"
+            authors = []
+            [dependencies]
+            b = { path = "../b" }
+        "#)
+        .file("a2/src/lib.rs", "")
+        .file("b/Cargo.toml", r#"
+            [package]
+            name = "b"
+            version = "0.0.1"
+            authors = []
+            [dependencies]
+            c = { path = "../c" }
+        "#)
+        .file("b/src/lib.rs", "")
+        .file("c/Cargo.toml", r#"
+            [package]
+            name = "c"
+            version = "0.0.1"
+            authors = []
+            [dependencies]
+            d = { path = "../d" }
+        "#)
+        .file("c/src/lib.rs", "")
+        .file("d/Cargo.toml", r#"
+            [package]
+            name = "d"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("d/src/lib.rs", "")
+        .file(".cargo/config", r#"
+            [build]
+            target-dir = "./target"
+        "#);
+    p.build();
+
+    assert_that(p.cargo("build").cwd(p.root().join("a1")),
+                execs().with_status(0).with_stderr(&format!("\
+[COMPILING] d v0.0.1 ({dir}/d)
+[COMPILING] c v0.0.1 ({dir}/c)
+[COMPILING] b v0.0.1 ({dir}/b)
+[COMPILING] a1 v0.0.1 ({dir}/a1)
+", dir = p.url())));
+    assert_that(p.cargo("build").cwd(p.root().join("a2")),
+                execs().with_status(0).with_stderr(&format!("\
+[COMPILING] a2 v0.0.1 ({dir}/a2)
+", dir = p.url())));
+});
