@@ -3,7 +3,6 @@ use std::io::prelude::*;
 use std::str;
 
 use support::{project, execs, basic_bin_manifest, basic_lib_manifest};
-use support::{COMPILING, RUNNING, DOCTEST};
 use support::paths::CargoPathExt;
 use hamcrest::{assert_that, existing_file, is_not};
 use cargo::util::process;
@@ -34,18 +33,16 @@ test!(cargo_test_simple {
                 execs().with_stdout("hello\n"));
 
     assert_that(p.cargo("test"),
-                execs().with_stdout(format!("\
-{} foo v0.5.0 ({})
-{} target[..]foo-[..]
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.5.0 ({})
+[RUNNING] target[..]foo-[..]", p.url()))
+                       .with_stdout("
 running 1 test
 test test_hello ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        COMPILING, p.url(),
-        RUNNING)));
+"));
 });
 
 test!(cargo_test_release {
@@ -81,36 +78,35 @@ test!(cargo_test_release {
         .file("bar/src/lib.rs", "pub fn bar() {}");
 
     assert_that(p.cargo_process("test").arg("-v").arg("--release"),
-                execs().with_stdout(format!("\
-{compiling} bar v0.0.1 ({dir})
-{running} [..] -C opt-level=3 [..]
-{compiling} foo v0.1.0 ({dir})
-{running} [..] -C opt-level=3 [..]
-{running} [..] -C opt-level=3 [..]
-{running} [..] -C opt-level=3 [..]
-{running} `[..]target[..]foo-[..]`
+                execs().with_stderr(format!("\
+[COMPILING] bar v0.0.1 ({dir}/bar)
+[RUNNING] [..] -C opt-level=3 [..]
+[COMPILING] foo v0.1.0 ({dir})
+[RUNNING] [..] -C opt-level=3 [..]
+[RUNNING] [..] -C opt-level=3 [..]
+[RUNNING] [..] -C opt-level=3 [..]
+[RUNNING] `[..]target[..]foo-[..]`
+[RUNNING] `[..]target[..]test-[..]`
+[DOCTEST] foo
+[RUNNING] `rustdoc --test [..]lib.rs[..]`", dir = p.url()))
+                       .with_stdout("
+running 1 test
+test test ... ok
+
+test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 1 test
 test test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} `[..]target[..]test-[..]`
-
-running 1 test
-test test ... ok
-
-test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
-
-{doctest} foo
-{running} `rustdoc --test [..]lib.rs[..]`
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-compiling = COMPILING, dir = p.url(), running = RUNNING, doctest = DOCTEST)));
+"));
 });
 
 test!(cargo_test_verbose {
@@ -122,18 +118,17 @@ test!(cargo_test_verbose {
         "#);
 
     assert_that(p.cargo_process("test").arg("-v").arg("hello"),
-        execs().with_stdout(format!("\
-{compiling} foo v0.5.0 ({url})
-{running} `rustc src[..]foo.rs [..]`
-{running} `[..]target[..]foo-[..] hello`
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.5.0 ({url})
+[RUNNING] `rustc src[..]foo.rs [..]`
+[RUNNING] `[..]target[..]foo-[..] hello`", url = p.url()))
+                       .with_stdout("
 running 1 test
 test test_hello ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        compiling = COMPILING, url = p.url(), running = RUNNING)));
+"));
 });
 
 test!(many_similar_names {
@@ -189,10 +184,11 @@ test!(cargo_test_failing_test {
                 execs().with_stdout("hello\n"));
 
     assert_that(p.cargo("test"),
-                execs().with_stdout_contains(format!("\
-{compiling} foo v0.5.0 ({url})
-{running} target[..]foo-[..]
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.5.0 ({url})
+[RUNNING] target[..]foo-[..]
+[ERROR] test failed", url = p.url()))
+                       .with_stdout_contains("
 running 1 test
 test test_hello ... FAILED
 
@@ -202,14 +198,14 @@ failures:
 <tab>thread 'test_hello' panicked at 'assertion failed: \
     `(left == right)` (left: \
     `\"hello\"`, right: `\"nope\"`)', src[..]foo.rs:12
-", compiling = COMPILING, url = p.url(), running = RUNNING))
-                    .with_stdout_contains("\
+")
+                       .with_stdout_contains("\
 failures:
     test_hello
 
 test result: FAILED. 0 passed; 1 failed; 0 ignored; 0 measured
 ")
-                    .with_status(101));
+                       .with_status(101));
 });
 
 test!(test_with_lib_dep {
@@ -246,31 +242,30 @@ test!(test_with_lib_dep {
         ");
 
     assert_that(p.cargo_process("test"),
-        execs().with_stdout(format!("\
-{} foo v0.0.1 ({})
-{running} target[..]baz-[..]
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({})
+[RUNNING] target[..]baz-[..]
+[RUNNING] target[..]foo[..]
+[DOCTEST] foo", p.url()))
+                       .with_stdout("
 running 1 test
 test bin_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]foo[..]
 
 running 1 test
 test lib_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 1 test
 test foo_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        COMPILING, p.url(), running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(test_with_deep_lib_dep {
@@ -313,27 +308,24 @@ test!(test_with_deep_lib_dep {
     p2.build();
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{compiling} bar v0.0.1 ({dir})
-{running} target[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ([..])
+[COMPILING] bar v0.0.1 ({dir})
+[RUNNING] target[..]
+[DOCTEST] bar", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test bar_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} bar
 
 running 1 test
 test bar_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       doctest = DOCTEST,
-                       dir = p.url())));
+"));
 });
 
 test!(external_test_explicit {
@@ -362,30 +354,29 @@ test!(external_test_explicit {
         "#);
 
     assert_that(p.cargo_process("test"),
-        execs().with_stdout(format!("\
-{} foo v0.0.1 ({})
-{running} target[..]foo-[..]
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({})
+[RUNNING] target[..]foo-[..]
+[RUNNING] target[..]test-[..]
+[DOCTEST] foo", p.url()))
+                       .with_stdout("
 running 1 test
 test internal_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]test-[..]
 
 running 1 test
 test external_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        COMPILING, p.url(), running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(external_test_implicit {
@@ -410,30 +401,29 @@ test!(external_test_implicit {
         "#);
 
     assert_that(p.cargo_process("test"),
-        execs().with_stdout(format!("\
-{} foo v0.0.1 ({})
-{running} target[..]external-[..]
-
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({})
+[RUNNING] target[..]external-[..]
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo", p.url()))
+                       .with_stdout("
 running 1 test
 test external_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]foo-[..]
 
 running 1 test
 test internal_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        COMPILING, p.url(), running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(dont_run_examples {
@@ -468,45 +458,40 @@ test!(pass_through_command_line {
 
     assert_that(p.cargo_process("test").arg("bar"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]foo-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test bar ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       doctest = DOCTEST,
-                       dir = p.url())));
+"));
 
     assert_that(p.cargo("test").arg("foo"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{running} target[..]foo-[..]
-
+                       .with_stderr("\
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo")
+                       .with_stdout("
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       running = RUNNING,
-                       doctest = DOCTEST)));
+"));
 });
 
 // Regression test for running cargo-test twice with
@@ -553,30 +538,29 @@ test!(lib_bin_same_name {
         ");
 
     assert_that(p.cargo_process("test"),
-        execs().with_stdout(format!("\
-{} foo v0.0.1 ({})
-{running} target[..]foo-[..]
+                execs().with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({})
+[RUNNING] target[..]foo-[..]
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo", p.url()))
+                       .with_stdout("
+running 1 test
+test [..] ... ok
+
+test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 1 test
 test [..] ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]foo-[..]
-
-running 1 test
-test [..] ... ok
-
-test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
-
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-        COMPILING, p.url(), running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(lib_with_standard_name {
@@ -605,32 +589,30 @@ test!(lib_with_standard_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} syntax v0.0.1 ({dir})
-{running} target[..]syntax-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] syntax v0.0.1 ({dir})
+[RUNNING] target[..]syntax-[..]
+[RUNNING] target[..]test-[..]
+[DOCTEST] syntax", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test foo_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]test-[..]
 
 running 1 test
 test test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} syntax
 
 running 1 test
 test foo_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       doctest = DOCTEST, dir = p.url())));
+"));
 });
 
 test!(lib_with_standard_name2 {
@@ -660,18 +642,16 @@ test!(lib_with_standard_name2 {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} syntax v0.0.1 ({dir})
-{running} target[..]syntax-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] syntax v0.0.1 ({dir})
+[RUNNING] target[..]syntax-[..]", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       dir = p.url())));
+"));
 });
 
 test!(lib_without_name {
@@ -700,16 +680,16 @@ test!(lib_without_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} syntax v0.0.1 ({dir})
-{running} target[..]syntax-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] syntax v0.0.1 ({dir})
+[RUNNING] target[..]syntax-[..]", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING, dir = p.url())));
+"));
 });
 
 test!(bin_without_name {
@@ -741,11 +721,11 @@ test!(bin_without_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(101)
-                       .with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                       .with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
-  binary target bin.name is required")));
+  binary target bin.name is required"));
 });
 
 test!(bench_without_name {
@@ -785,11 +765,11 @@ test!(bench_without_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(101)
-                       .with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                       .with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
-  bench target bench.name is required")));
+  bench target bench.name is required"));
 });
 
 test!(test_without_name {
@@ -828,11 +808,11 @@ test!(test_without_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(101)
-                       .with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                       .with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
-  test target test.name is required")));
+  test target test.name is required"));
 });
 
 test!(example_without_name {
@@ -871,11 +851,11 @@ test!(example_without_name {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(101)
-                       .with_stderr(&format!("\
-failed to parse manifest at `[..]`
+                       .with_stderr("\
+[ERROR] failed to parse manifest at `[..]`
 
 Caused by:
-  example target example.name is required")));
+  example target example.name is required"));
 });
 
 test!(bin_there_for_integration {
@@ -950,46 +930,43 @@ test!(test_dylib {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} bar v0.0.1 ({dir})
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]foo-[..]
+                       .with_stderr(&format!("\
+[COMPILING] bar v0.0.1 ({dir}/bar)
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]foo-[..]
+[RUNNING] target[..]test-[..]", dir = p.url()))
+                       .with_stdout("
+running 1 test
+test foo ... ok
+
+test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]test-[..]
-
-running 1 test
-test foo ... ok
-
-test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
-
-",
-                       compiling = COMPILING, running = RUNNING,
-                       dir = p.url())));
+"));
     p.root().move_into_the_past().unwrap();
     assert_that(p.cargo("test"),
                 execs().with_status(0)
-                       .with_stdout(format!("\
-{running} target[..]foo-[..]
+                       .with_stderr("\
+[RUNNING] target[..]foo-[..]
+[RUNNING] target[..]test-[..]")
+                       .with_stdout("
+running 1 test
+test foo ... ok
+
+test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]test-[..]
-
-running 1 test
-test foo ... ok
-
-test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
-
-",
-                       running = RUNNING)));
+"));
 
 });
 
@@ -1010,45 +987,40 @@ test!(test_twice_with_build_cmd {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]foo-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       doctest = DOCTEST,
-                       dir = p.url())));
+"));
 
     assert_that(p.cargo("test"),
                 execs().with_status(0)
-                       .with_stdout(format!("\
-{running} target[..]foo-[..]
-
+                       .with_stderr("\
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo")
+                       .with_stdout("
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       running = RUNNING,
-                       doctest = DOCTEST)));
+"));
 });
 
 test!(test_then_build {
@@ -1066,25 +1038,22 @@ test!(test_then_build {
 
     assert_that(p.cargo_process("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]foo-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]foo-[..]
+[DOCTEST] foo", dir = p.url()))
+                       .with_stdout("
 running 1 test
 test foo ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-",
-                       compiling = COMPILING, running = RUNNING,
-                       doctest = DOCTEST,
-                       dir = p.url())));
+"));
 
     assert_that(p.cargo("build"),
                 execs().with_status(0)
@@ -1106,10 +1075,9 @@ test!(test_no_run {
 
     assert_that(p.cargo_process("test").arg("--no-run"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
 ",
-                       compiling = COMPILING,
                        dir = p.url())));
 });
 
@@ -1132,22 +1100,18 @@ test!(test_run_specific_bin_target {
         .file("src/bin1.rs", "#[test] fn test1() { }")
         .file("src/bin2.rs", "#[test] fn test2() { }");
 
-    let expected_stdout = format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]bin2-[..]
-
+    assert_that(prj.cargo_process("test").arg("--bin").arg("bin2"),
+                execs().with_status(0)
+                       .with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]bin2-[..]", dir = prj.url()))
+                       .with_stdout("
 running 1 test
 test test2 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-       compiling = COMPILING,
-       running = RUNNING,
-       dir = prj.url());
-
-    assert_that(prj.cargo_process("test").arg("--bin").arg("bin2"),
-        execs().with_status(0).with_stdout(&expected_stdout));
+"));
 });
 
 test!(test_run_specific_test_target {
@@ -1163,22 +1127,18 @@ test!(test_run_specific_test_target {
         .file("tests/a.rs", "#[test] fn test_a() { }")
         .file("tests/b.rs", "#[test] fn test_b() { }");
 
-    let expected_stdout = format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]b-[..]
-
+    assert_that(prj.cargo_process("test").arg("--test").arg("b"),
+                execs().with_status(0)
+                       .with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]b-[..]", dir = prj.url()))
+                       .with_stdout("
 running 1 test
 test test_b ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-",
-       compiling = COMPILING,
-       running = RUNNING,
-       dir = prj.url());
-
-    assert_that(prj.cargo_process("test").arg("--test").arg("b"),
-        execs().with_status(0).with_stdout(&expected_stdout));
+"));
 });
 
 test!(test_no_harness {
@@ -1203,11 +1163,10 @@ test!(test_no_harness {
 
     assert_that(p.cargo_process("test").arg("--").arg("--nocapture"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]bar-[..]
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]bar-[..]
 ",
-                       compiling = COMPILING, running = RUNNING,
                        dir = p.url())));
 });
 
@@ -1258,56 +1217,53 @@ test!(selective_testing {
     println!("d1");
     assert_that(p.cargo("test").arg("-p").arg("d1"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} d1 v0.0.1 ({dir})
-{running} target[..]d1-[..]
+                       .with_stderr(&format!("\
+[COMPILING] d1 v0.0.1 ({dir}/d1)
+[RUNNING] target[..]d1-[..]
+[RUNNING] target[..]d1-[..]", dir = p.url()))
+                       .with_stdout("
+running 0 tests
+
+test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]d1-[..]
-
-running 0 tests
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
-
-", compiling = COMPILING, running = RUNNING,
-   dir = p.url())));
+"));
 
     println!("d2");
     assert_that(p.cargo("test").arg("-p").arg("d2"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} d2 v0.0.1 ({dir})
-{running} target[..]d2-[..]
+                       .with_stderr(&format!("\
+[COMPILING] d2 v0.0.1 ({dir}/d2)
+[RUNNING] target[..]d2-[..]
+[RUNNING] target[..]d2-[..]", dir = p.url()))
+                       .with_stdout("
+running 0 tests
+
+test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]d2-[..]
-
-running 0 tests
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
-
-", compiling = COMPILING, running = RUNNING,
-   dir = p.url())));
+"));
 
     println!("whole");
     assert_that(p.cargo("test"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} target[..]foo-[..]
-
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] target[..]foo-[..]", dir = p.url()))
+                       .with_stdout("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING,
-   dir = p.url())));
+"));
 });
 
 test!(almost_cyclic_but_not_quite {
@@ -1456,22 +1412,21 @@ test!(selective_testing_with_docs {
 
     assert_that(p.cargo("test").arg("-p").arg("d1"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} d1 v0.0.1 ({dir})
-{running} target[..]deps[..]d1[..]
+                       .with_stderr(&format!("\
+[COMPILING] d1 v0.0.1 ({dir}/d1)
+[RUNNING] target[..]deps[..]d1[..]
+[DOCTEST] d1", dir = p.url()))
+                       .with_stdout("
+running 0 tests
+
+test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
+
 
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} d1
-
-running 0 tests
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
-
-", compiling = COMPILING, running = RUNNING, dir = p.url(),
-   doctest = DOCTEST)));
+"));
 });
 
 test!(example_bin_same_name {
@@ -1487,11 +1442,11 @@ test!(example_bin_same_name {
 
     assert_that(p.cargo_process("test").arg("--no-run").arg("-v"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ({dir})
-{running} `rustc [..]`
-{running} `rustc [..]`
-", compiling = COMPILING, running = RUNNING, dir = p.url())));
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[RUNNING] `rustc [..]`
+[RUNNING] `rustc [..]`
+", dir = p.url())));
 
     assert_that(&p.bin("foo"), is_not(existing_file()));
     assert_that(&p.bin("examples/foo"), existing_file());
@@ -1501,11 +1456,12 @@ test!(example_bin_same_name {
 
     assert_that(p.cargo("run"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ([..])
-{running} [..]
+                       .with_stderr("\
+[COMPILING] foo v0.0.1 ([..])
+[RUNNING] [..]")
+                       .with_stdout("\
 bin
-", compiling = COMPILING, running = RUNNING)));
+"));
     assert_that(&p.bin("foo"), existing_file());
 });
 
@@ -1558,13 +1514,13 @@ test!(example_with_dev_dep {
 
     assert_that(p.cargo_process("test").arg("-v"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
+                       .with_stderr("\
 [..]
 [..]
 [..]
 [..]
-{running} `rustc [..] --crate-name ex [..] --extern a=[..]`
-", running = RUNNING)));
+[RUNNING] `rustc [..] --crate-name ex [..] --extern a=[..]`
+"));
 });
 
 test!(bin_is_preserved {
@@ -1600,11 +1556,11 @@ test!(bad_example {
 
     assert_that(p.cargo_process("run").arg("--example").arg("foo"),
                 execs().with_status(101).with_stderr("\
-no example target named `foo`
+[ERROR] no example target named `foo`
 "));
     assert_that(p.cargo_process("run").arg("--bin").arg("foo"),
                 execs().with_status(101).with_stderr("\
-no bin target named `foo`
+[ERROR] no bin target named `foo`
 "));
 });
 
@@ -1627,22 +1583,23 @@ test!(doctest_feature {
         "#);
 
     assert_that(p.cargo_process("test").arg("--features").arg("bar"),
-                execs().with_status(0).with_stdout(format!("\
-{compiling} foo [..]
-{running} target[..]foo[..]
-
+                execs().with_status(0)
+                       .with_stderr("\
+[COMPILING] foo [..]
+[RUNNING] target[..]foo[..]
+[DOCTEST] foo")
+                       .with_stdout("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 1 test
 test foo_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(dashes_to_underscores {
@@ -1710,15 +1667,15 @@ test!(filter_no_doc_tests {
         .file("tests/foo.rs", "");
 
     assert_that(p.cargo_process("test").arg("--test=foo"),
-                execs().with_stdout(format!("\
-{compiling} foo v0.0.1 ([..])
-{running} target[..]debug[..]foo[..]
-
+                execs().with_stderr("\
+[COMPILING] foo v0.0.1 ([..])
+[RUNNING] target[..]debug[..]foo[..]")
+                       .with_stdout("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING)));
+"));
 });
 
 test!(dylib_doctest {
@@ -1742,16 +1699,16 @@ test!(dylib_doctest {
         "#);
 
     assert_that(p.cargo_process("test"),
-                execs().with_stdout(format!("\
-{compiling} foo v0.0.1 ([..])
-{doctest} foo
-
+                execs().with_stderr("\
+[COMPILING] foo v0.0.1 ([..])
+[DOCTEST] foo")
+                       .with_stdout("
 running 1 test
 test foo_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, doctest = DOCTEST)));
+"));
 });
 
 test!(dylib_doctest2 {
@@ -1808,23 +1765,23 @@ test!(cyclic_dev_dep_doc_test {
             extern crate foo;
         "#);
     assert_that(p.cargo_process("test"),
-                execs().with_stdout(format!("\
-{compiling} foo v0.0.1 ([..])
-{compiling} bar v0.0.1 ([..])
-{running} target[..]foo[..]
-
+                execs().with_stderr("\
+[COMPILING] foo v0.0.1 ([..])
+[COMPILING] bar v0.0.1 ([..])
+[RUNNING] target[..]foo[..]
+[DOCTEST] foo")
+                       .with_stdout("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 1 test
 test _0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", compiling = COMPILING, running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(dev_dep_with_build_script {
@@ -1899,34 +1856,35 @@ test!(no_fail_fast {
         "#);
     assert_that(p.cargo_process("test").arg("--no-fail-fast"),
                 execs().with_status(101)
-                       .with_stdout_contains(format!("\
-{compiling} foo v0.0.1 ([..])
-{running} target[..]foo[..]
-
+                       .with_stderr_contains("\
+[COMPILING] foo v0.0.1 ([..])
+[RUNNING] target[..]foo[..]
+[RUNNING] target[..]test_add_one[..]")
+                       .with_stdout_contains("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
 
-{running} target[..]test_add_one[..]
-", compiling = COMPILING, running = RUNNING))
-                       .with_stdout_contains(format!("\
+")
+                       .with_stderr_contains("\
+[RUNNING] target[..]test_sub_one[..]
+[DOCTEST] foo")
+                       .with_stdout_contains("\
 test result: FAILED. 1 passed; 1 failed; 0 ignored; 0 measured
 
-{running} target[..]test_sub_one[..]
 
 running 1 test
 test sub_one_test ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-{doctest} foo
 
 running 1 test
 test sub_one_0 ... ok
 
 test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
 
-", running = RUNNING, doctest = DOCTEST)))
+"))
 });
 
 test!(test_multiple_packages {
@@ -1973,20 +1931,20 @@ test!(test_multiple_packages {
 
     assert_that(p.cargo("test").arg("-p").arg("d1").arg("-p").arg("d2"),
                 execs().with_status(0)
-                       .with_stdout_contains(&format!("\
-{running} target[..]debug[..]d1-[..]
-
+                       .with_stderr_contains("\
+[RUNNING] target[..]debug[..]d1-[..]")
+                       .with_stdout_contains("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
-", running = RUNNING))
-                       .with_stdout_contains(&format!("\
-{running} target[..]debug[..]d2-[..]
-
+")
+                       .with_stderr_contains("\
+[RUNNING] target[..]debug[..]d2-[..]")
+                       .with_stdout_contains("
 running 0 tests
 
 test result: ok. 0 passed; 0 failed; 0 ignored; 0 measured
-", running = RUNNING)));
+"));
 });
 
 test!(bin_does_not_rebuild_tests {
@@ -2011,11 +1969,11 @@ test!(bin_does_not_rebuild_tests {
 
     assert_that(p.cargo("test").arg("-v").arg("--no-run"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} foo v0.0.1 ([..])
-{running} `rustc src[..]main.rs [..]`
-{running} `rustc src[..]main.rs [..]`
-", compiling = COMPILING, running = RUNNING)));
+                       .with_stderr("\
+[COMPILING] foo v0.0.1 ([..])
+[RUNNING] `rustc src[..]main.rs [..]`
+[RUNNING] `rustc src[..]main.rs [..]`
+"));
 });
 
 test!(selective_test_wonky_profile {
@@ -2070,9 +2028,80 @@ test!(selective_test_optional_dep {
 
     assert_that(p.cargo("test").arg("-v").arg("--no-run")
                  .arg("--features").arg("a").arg("-p").arg("a"),
-                execs().with_status(0).with_stdout(&format!("\
-{compiling} a v0.0.1 ([..])
-{running} `rustc a[..]src[..]lib.rs [..]`
-{running} `rustc a[..]src[..]lib.rs [..]`
-", compiling = COMPILING, running = RUNNING)));
+                execs().with_status(0).with_stderr("\
+[COMPILING] a v0.0.1 ([..])
+[RUNNING] `rustc a[..]src[..]lib.rs [..]`
+[RUNNING] `rustc a[..]src[..]lib.rs [..]`
+"));
+});
+
+test!(only_test_docs {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/lib.rs", r#"
+            #[test]
+            fn foo() {
+                let a: u32 = "hello";
+            }
+
+            /// ```
+            /// println!("ok");
+            /// ```
+            pub fn bar() {
+            }
+        "#)
+        .file("tests/foo.rs", "this is not rust");
+    p.build();
+
+    assert_that(p.cargo("test").arg("--doc"),
+                execs().with_status(0)
+                       .with_stderr(&format!("\
+[COMPILING] foo v0.0.1 ([..])
+[DOCTEST] foo"))
+                       .with_stdout("
+running 1 test
+test bar_0 ... ok
+
+test result: ok.[..]
+
+"));
+});
+
+test!(test_panic_abort_with_dep {
+    if !::is_nightly() {
+        return
+    }
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies]
+            bar = { path = "bar" }
+
+            [profile.dev]
+            panic = 'abort'
+        "#)
+        .file("src/lib.rs", r#"
+            extern crate bar;
+
+            #[test]
+            fn foo() {}
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [package]
+            name = "bar"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("bar/src/lib.rs", "");
+    assert_that(p.cargo_process("test").arg("-v"),
+                execs().with_status(0));
 });
